// Fichier Barillet.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                En-tte     de la librairie           *
 *					de la carte barillet                *
 *                                                      *
 * .Date de cration : 06/02/07                         *
 * .Auteur : Xavier Maquignon                           *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte barillet     *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/


#define 	ADR_I2C_BARILLET				0x86		// Adresse IC de la carte barillet

#define		JOUER_BLEU						0xE0		// initialisation du barillet
#define		BARI_VIDANGE					0xD2		// vidange du barillet

#define		BARI_DEPOT_CHEZ_NOUS			0x40		// dpose d'une balle bleue  droite
#define		BARI_DEPOT_CHEZ_ADV				0x2C		// dpose d'une balle blanche  droite


#define		BARI_DEPOT_BLEUE				0x56		// dpose d'une balle bleue  droite
#define		BARI_DEPOT_BLANCHE				0xB8		// dpose d'une balle blanche  droite
#define		BARI_DEPOT_ROUGE				0xA6		// dpose d'une balle rouge  droite

#define		BARI_SERVO_GAUCHE				0x47		// dpose d'une balle bleue  gauche
#define		BARI_SERVO_DROITE				0x23		// dpose d'une balle blanche  gauche

#define		BARI_TIR_BLEUE					0x7E		// tir d'une balle bleue
#define		BARI_TIR_BLANCHE				0x98		// tir d'une balle blanche
#define		BARI_TIR_ROUGE					0x46		// tir d'une balle rouge
#define		BARI_STOP						0x32		// barillet en position intermdiaire

#define		NONE							0
#define 	BLEUE							1
#define 	BLANCHE							2
#define 	ROUGE							3
#define 	TOUTES							4

#define			TEMPO_INTER_REQUETES_I2C	Delay10KTCYx(20);	// Tempo entre deux requetes IC

typedef struct {
	char slot[5];	// ce qu'il y a dans chaque slot
	char avale;		// est-ce qu'on est en train d'avaler une balle
	char busy; 		// est-ce qu'on est busy ?
} TEtatBarillet;



// --------------------------------------------------------------------------
/* Place le servo de dpose pour faire un dpose vers la gauche
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Servo_Gauche_Barillet(void);




// --------------------------------------------------------------------------
/* Place le servo de dpose pour faire un dpose vers la droite
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Servo_Droite_Barillet(void);




// --------------------------------------------------------------------------
/* Vidange intgralement le barillet dans l'ordre le plus rapide
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Vidange_Barillet( void );



// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
/* Envoie l'ordre de stopper le barillet
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Stop_Barillet( void );


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
/* Informe le barillet que l'on joue la couleur bleu
 * Arguments : aucun
 * Note : par dafaut, le barillet joue la couleur rouge
 *
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Jouer_Bleu_Barillet(void);

// --------------------------------------------------------------------------
/* ralise une dpose de notre barillet de notre ct
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int DeposeBalle_Chez_Nous_Barillet( void );


// --------------------------------------------------------------------------
/* ralise une dpose de notre barillet chez l'adversaire
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int DeposeBalle_Adverse_Barillet( void );



// --------------------------------------------------------------------------
/* Dpose une balle de la couleur passe en paramtre
 * Arguments : 
 *  1 ou BLEUE	 : dpose une balle bleue
 *  2 ou BLANCHE : dpose une balle blanche
 *  3 ou ROUGE	 : dpose une balle rouge
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int DeposeBalle_Barillet( char couleur );


// --------------------------------------------------------------------------
/* Tire une balle de la couleur passe en paramtre
 * Arguments : 
 *  1 ou BLEUE	 : tire une balle bleue
 *  2 ou BLANCHE : tire une balle blanche
 *  3 ou ROUGE	 : tire une balle rouge
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int TirBalle_Barillet( char couleur );






// --------------------------------------------------------------------------
/* Lit l'tat et le contenu du barillet
 * Arguments : pointeur vers une struture de type TEtatBarillet
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Etat_Barillet( TEtatBarillet* etat );



// --------------------------------------------------------------------------
/* Fonction qui indique si le barillet est occup ou non
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass, il n'est pas occup
 *   1 : le barillet est occup
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Barillet_Busy(void);



// --------------------------------------------------------------------------
/* Fonction qui indique si le barillet est en train d'avaler une balle
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass, il n'y a pas de balle
 *   1 : une balle est en cours de traitement
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Barillet_Avale(void);



// --------------------------------------------------------------------------
/* Effectue un chek-up du barillet
 * Vidange le barillet   								(erreurs 1)
 * Lit le contenu et l'tat du barillet				 	(erreurs 2)
 * Vrifie que le barillet est vide						(erreurs 3)
 * Pas de problme : on retourne 0 
*/
char Check_Up_Barillet(unsigned char NotreCouleur);



// --------------------------------------------------------------------------
/* Renvoie le nombre de balles d'une couleurs dans le barillet
 * 
 * Pas de problme : on retourne le nombre de balles
 * -1 problme sur l'I2C
*/
char NbBalles_Barillet(unsigned char Couleur);
