// Fichier Horloge.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - Chronomtre pour mesurer les 90secondes       *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/


#include <p18f452.h>				// Pour les registres 
#include "mystdio.h"				// Pour les printf ( la fin du match)
#include <delays.h>					// Pour faire clignoter l'cran  la fin du match
#include <timers.h>					// Pour le timer

#include "Horloge.h"				// Prototypes des fonctions
#include "lcd.h"					// Pour l'afficheur LCD
#include "led.h"					// Pour les leds
#include "propulsion.h"				// Pour arrter les moteurs  la fin du match
#include "rouleaux.h"				// Pour arrter les rouleaux  la fin du match
#include "serial.h"					// Pour forcer l'initialisation de la liaison srie
#include "tourelle.h"				// Pour stopper la tourelle
#include "canon.h"					// Pour arrter le canon
#include "barillet.h"				// Pour arrter le barillet



volatile int 			Nb_Interrupts=0;		// Compte les interruptions


// --------------------------------------------------------------------------
/* Arrte le robot  la fin du match
 * Coupe les moteurs et les rouleaux
 * Arguments : aucun 
 * 
 * Paramtre renvoy : aucun
*/
void Fin_Match(void)
{
	LCD_Clear();								// Efface l'cran	
	print_str ("90s : Fin du match\n");			// Affiche le message





	print_str ("Propulsion      ");
	while (Stop_Propulsion())					// Tant que l'arrt des moteurs n'est pas valide
	{
		Serial_ReStart();						// Vide le buffer au cas ou on serait en train de transmettre une commande
		Delay10KTCYx(1);						// On attends un peu
	}
	print_str ("[OK]");							// Les moteurs sont arrts, on affiche OK




	print_str ("Rouleaux droit  ");				// Arrt du rouleau droit
	while (Mesure_Rouleau_Droit()!=0)			// Tant que le rouleau n'est pas stopp
	{
		Delay10KTCYx(1);						// On attends un peu
		Commande_Rouleau_Droit(0);				// On arrete le rouleau 
	}
	print_str ("[OK]");							// Le rouleau est arrt, on affiche OK




	print_str ("Rouleaux gauche ");				// Arrt du rouleau gauche
	while (Mesure_Rouleau_Gauche()!=0)			// Tant que le rouleau n'est pas stopp
	{
		Delay10KTCYx(1);						// On attends un peu
		Commande_Rouleau_Gauche(0);				// On arrete le rouleau
	}
	print_str ("[OK]");							// Le rouleau est arrt, on affiche OK

	
	
	print_str ("Tourelle        ");
	Stop_Tourelle();
	print_str ("[OK]");							// Le rouleau est arrt, on affiche OK


	print_str ("Canon           ");
	Commande_Canon(0);
	print_str ("[OK]");							// Le rouleau est arrt, on affiche OK

	print_str ("Barillet        ");
	while (Stop_Barillet());
	print_str ("[OK]");							// Le barillet est arrt, on affiche OK


	LCD_BackLight(LCD_BL_OFF);					// Eteinds le rtro-clairage
	while (1);
}




// --------------------------------------------------------------------------
/* Initialise et dmarre le chronomtre de 90 secondes
 * Arguments : aucun 
 * 
 * Paramtre renvoy : aucun
*/
void StartHorloge(void)							
{
	Nb_Interrupts=0;				// Met le compteur d'interruptions  zro
	
	OpenTimer0(	TIMER_INT_ON &		// Interruption sur timer 0
				T0_16BIT &			// Timer en 16 bits
				T0_SOURCE_INT &		// Source interne (quartz)
				T0_PS_1_1			// Pr-scaler 1:1
				);
	WriteTimer0 (VAL_TIMER0);		// Charge le timer	
	INTCONbits.GIE	= 1; 			// Autorise les interruptions... et c'est parti pour 90s
}




// --------------------------------------------------------------------------
/* Retourne le nombre de centime de secondes coule depuis le dbut du match
 * Arguments : aucun 
 * 
 * Paramtre renvoy : centime de secondes
*/
int Cent_depuis_debut (void)
{
	int Ret;						// Copie de la valeur pour viter les problmes d'interruption
	INTCONbits.TMR0IE=0;			// Interdit l'interruption
	Ret=Nb_Interrupts;				// Copie la valeur
	INTCONbits.TMR0IE=1;			// Rautorise l'interruption
	return (Ret);					// Retourne la valeur			
}



// --------------------------------------------------------------------------
/* Retourne le nombre de secondes coul depuis le dbut du match
 * Arguments : aucun 
 * 
 * Paramtre renvoy : secondes
*/
int Sec_depuis_debut (void)
{
	int Ret;						// Copie de la valeur pour viter les problmes d'interruption
	INTCONbits.TMR0IE=0;			// Interdit l'interruption
	Ret=Nb_Interrupts;				// Copie la valeur
	INTCONbits.TMR0IE=1;			// Rautorise l'interruption
	return (Ret/100);				// Convertit en seconde et retourne la valeur
}





// --------------------------------------------------------------------------
/* Vecteur d'interruption
*/
#pragma code InterruptVectorHigh = 0x08   //Priorit basse
void InterruptVectorHigh(void)  
{	
	_asm
		goto ServiceInterruptHigh	//Saut  la routine d'interruption haute
	_endasm
}
#pragma code




// --------------------------------------------------------------------------
/* Routine d'interruptions
 *  Met  jour le compteur
 *  Si la fin du match est atteinte, on appelle la fonction Fin_Match
 *  pour  stopper le robot
*/
#pragma interrupt ServiceInterruptHigh
void ServiceInterruptHigh(void)
{	
	if(INTCONbits.TMR0IF) 					//Dbordement du timer0
	{	
		INTCONbits.TMR0IF=0;				// Rarme l'interruptino
		WriteTimer0 (VAL_TIMER0);			// Recharge le timer
		Nb_Interrupts++;					// Une nouvelle interruption


		if (Nb_Interrupts>9000)				// On a pass les 90secondes
		{
			Fin_Match();
		}		
	}
}
#pragma code
