// Fichier Led.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 27/01/08                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/


#include "mystdio.h"
#include <delays.h>


#include "LCD.h"
#include "Joystick.h"
#include "CAN.h"
#include "Batteries.h"
#include "IO.h"
#include "Telemetres.h"
#include "Propulsion.h"
#include "I2C_Master.h"
#include "led.h"
#include "StartUp.h"
#include "Rouleaux.h"
#include "Serial.h"
#include "HighLevel.h"
#include "Barillet.h"
#include "Tourelle.h"
#include "Canon.h"



// --------------------------------------------------------------------------
/* Fonction Start_Up
 * Initialise tous les priphriques du robot
 *  - la led rouge
 *  - L'cran LCD
 *  - Les convertisseurs Analogiques/Numriques
 *  - Les entres sorties (Switch G/D & Jack)
 *  - La liaison srie
 *  - Le module I2C 
*/

void Init_Peripheriques (void)
{
	//ConfigureProcesseur();

	Led_Init();		// Initalisation de la led et des switchs
	LCD_Init();		// Afficheur LCD
	CAN_Init();		// Convertisseurs Analogiques/Numriques
	IO_Init();		// Initialisation des ports pour les switchs et le jack
	Serial_Init();	// Initialisation de la liaison srie
	I2C_Init();		// Initialisation du bus I2C	
}	





// --------------------------------------------------------------------------
/* Scanne tous les priphriques et vrifie leur fonctionnement
 *  - la fonction retourne la couleur joue
 *  - Teste le fonctionnement des rouleaux dans les deux sens de rotation 
 * La fonction reoit la couleur joue (pour l'initialisation du barillet)
 *  
 * La fonction retourne 1 si le robot adverse est potentiellement dangeureux
 * La fonction retourne 0 si le robot adverse ne prsente pas de dangers pour nos balles
*/
unsigned char Check_Up (unsigned char CouleurJoue)
{
	char 			NoError;
	unsigned char 	BadRobot=2;						// Passe  un si l'adversaire prsente un danger pour notre conteneur
	
	// Pour l'cran LCD
//	stdout = _H_USER;								// Flux de sortie = afficheur LCD
	LCD_BackLight(LCD_BL_ON);						// Allume le rtro-clairage

	
	// Message d'acceuil
		
	print_str (MESSAGE_ACCUEIL);					// Affiche le message d'acceuil
	if( CouleurJoue==ROUGE ){
		print_str ("   :::  ROUGE  :::  ");				// Affiche le message d'acceuil
		Servo_Droite_Barillet();						// Place le servo de dpose du ct bleu
	}
	else{
		print_str ("   :::  BLEU   :::  ");				// Affiche le message d'acceuil
		Servo_Gauche_Barillet();						// Place le servo de dpose du ct bleu
	}
	
	// Slectionne si l'adversaire peut enlever nos balles
	
	LCD_GotoXY(0,2);
	print_str ("Bad bot     Good bot");
	
	
	while (!Joy_Sw() || BadRobot==2)
	{
		if (Joy_X()>850)
		{
			LCD_GotoXY(0,2);
			print_str ("            Good bot");
			BadRobot=0;
		}
		if (Joy_X()<250)
		{
			LCD_GotoXY(0,2);
			print_str ("Bad bot             ");
			BadRobot=1;
		}
		
	}
	
// Vrification de l'absence du Jack
	
	print_str ("Absence jack    ");					// Vrifie l'absence du jack
	if (!IO_Jack()) 								// Le jack est-il prsent
		print_str ("[OK]");							// Pas d'erreur
	else 
		Error_Blink(1);								// Jack prsent
		
	
// Vrification de la batterie logique
	
	print_str ("Bat logique     ");					// Mesure la batterie puissance
	if (BatterieLogiqueVolt()>CHECK_UP_SEUIL_12V) 	// Assez de batterie ?
		print_str ("[OK]");							// Pas d'erreur
	else Error_Blink(1);							// Batteries faibles
	
	
// Vrification de la batterie puissance	
		
	print_str ("Bat puissance   ");					// Mesure la batterie puissance
	if (BatteriePuissanceVolt()>CHECK_UP_SEUIL_18V) // Assez de batterie ?
		print_str ("[OK]");							// Pas d'erreur
	else Error_Blink(1);							// Batteries faibles


// Enlever cale

	print_str ("Enlever la cale     ");
	Delay10KTCYx(200);					  

// Verification des switchs

	NoError=0;
	print_str ("switch droit    ");				// Test du rouleau droit 
	if(!IO_Sw_Droit()){								//si switch pas enfonc
		while(!IO_Sw_Droit()){
			if( Joy_Sw() ){
				NoError=1;
				break;	
			}			
		}
		if (NoError==0) print_str ("[OK]");				// Pas d'erreur
		else Error_Blink(NoError);						// Affiche le message d'erreur						
	}else{
		NoError=2;
		Error_Blink(NoError);						// Affiche le message d'erreur						
	}
	Delay10KTCYx(150);

	NoError=0;
	print_str ("switch gauche   ");				// Test du rouleau droit 
	if(!IO_Sw_Gauche()){						// si switch pas enfonc
		while(!IO_Sw_Gauche()){
			if( Joy_Sw() ){
				NoError=1;
				break;	
			}			
		}
		if (NoError==0) print_str ("[OK]");				// Pas d'erreur
		else Error_Blink(NoError);						// Affiche le message d'erreur						
	}else{
		NoError=2;
		Error_Blink(NoError);						// Affiche le message d'erreur						
	}


// Verification de la carte propulsion

	NoError=0;										// A priori, pas d'erreurs
	print_str ("Propulsion      ");					// Test du rouleau droit 
	NoError= Check_Up_Propulsion(CouleurJoue);		// Check-Up de la carte propulsion
	Stop_Propulsion();								// Arrte les moteurs (en cas de problme par exemple)
	if (NoError==0) print_str ("[OK]");				// Pas d'erreur
	else Error_Blink(NoError);						// Affiche le message d'erreur						
	
	
	
	
// Verification du canon

	NoError=0;										// A priori, pas d'erreurs
	print_str ("Canon           ");					// Test du rouleau droit 
	NoError = Check_Up_Canon();						// Check-Up du rouleau droit
	Commande_Canon(0);								// Arrte le rouleau
	if (NoError==0) print_str ("[OK]");				// Pas d'erreur
	else Error_Blink(NoError);						// Affiche le message d'erreur


// Verification du rouleau droit

	NoError=0;										// A priori, pas d'erreurs
	print_str ("Rouleau droit   ");					// Test du rouleau droit 
	NoError= Check_Up_Rouleau_Droit();				// Check-Up du rouleau droit
	Commande_Rouleau_Droit(0);						// Arrte le rouleau
	if (NoError==0) print_str ("[OK]");				// Pas d'erreur
	else Error_Blink(NoError);						// Affiche le message d'erreur


// Verification du rouleau gauche

	NoError=0;										// A priori, pas d'erreurs
	print_str ("Rouleau gauche  ");					// Test du rouleau droit 
	NoError= Check_Up_Rouleau_Gauche();				// Check-Up du rouleau droit
	Commande_Rouleau_Gauche(0);						// Arrte le rouleau
	if (NoError==0) print_str ("[OK]");				// Pas d'erreur
	else Error_Blink(NoError);						// Affiche le message d'erreur

//Initialisation de la tourelle

	NoError = 0;									// A priori, pas d'erreurs
	print_str("Tourelle        ");					// Test de la tourelle
	NoError = Check_Up_Tourelle(CouleurJoue);		// Check-Up de la tourelle
	if (NoError==0) print_str ("[OK]");				// Pas d'erreur
	else Error_Blink(NoError);						// Affiche le message d'erreur

//Initialisation du barillet

	NoError = 0;									// A priori, pas d'erreurs
	print_str("Barillet        ");					// Test du barillet
	NoError = Check_Up_Barillet(CouleurJoue);		// Check-Up du barillet
	if (NoError==0) print_str ("[OK]");				// Pas d'erreur
	else Error_Blink(NoError);						// Affiche le message d'erreur
	
	print_str (":: Fin du CheckUp ::");
	return BadRobot;
}	




// --------------------------------------------------------------------------
/* Affiche un message d'erreur jusqu' un appui sur le joystick
 * Arguments : 
 *  Numro d'erreur
 * 
 * Paramtre renvoy : aucun
*/
void Error_Blink (unsigned char Error)
{	
	print_str ("Err");							// Affiche le message d'erreur
	print_int( Error );
	
	while (!Joy_Sw())								// Tant que le bouton n'est pas appuy
	{
		LCD_BackLight(LCD_BL_OFF);					// Eteinds le rtro-clairage
		Delay10KTCYx(40);
		LCD_BackLight(LCD_BL_ON);					// Allume le rtro-clairage
		Delay10KTCYx(40);		
	}

	Joy_Click();									// Attends un second click pour ngliger l'erreur
}	