// Fichier Robot.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>		// Librairie du PIC utilis
#include <delays.h>			// Delais pour le check-up


#include "I2C_Master.h"		// Librairie pour la liaison srie
#include "Tourelle.h"		// Entte de la librairie de gestion du robot
#include "Propulsion.h"		// Entte pour la carte propulsion
#include "Joystick.h"
#include "mystdio.h"
#include "Constante.h"



// --------------------------------------------------------------------------
/* Dmarre la tourelle la tourelle
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Start_Tourelle(void)
{
	return (I2C_Send(ADR_I2C_TOURELLE,0x00));				// Retourne le rsultat de l'envoie sur l'I2C
}	



// --------------------------------------------------------------------------
/* Stoppe la tourelle
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Stop_Tourelle(void)
{
	return (I2C_Send(ADR_I2C_TOURELLE,0x53));				// Retourne le rsultat de l'envoie sur l'I2C
}		



// --------------------------------------------------------------------------
/* Retourne l'angle avec le robot adverse
 *  entre 0 et 255 si tout s'est bien pass
 *  -1 en cas de problme dans la transmission
*/
int Angle_Robot_Adverse(void) 
{
	int ret,i=1;
	do 
	{
		Delay100TCYx(i);
		ret=I2C_Receive(ADR_I2C_TOURELLE);					// Lit l'angle du robot adverse via l'IC
	}
	while (ret<0 && i++<10);
}




// --------------------------------------------------------------------------
/* Retourne la vitesse angulaire du robot adverse
 *  entre -128 et 128 si tout s'est bien pass
 *  >0 Le robot adverse se dplace dans le sens trigo
 *  <0 Le robot adverse se dplace dans le sens horaire
 *  0xFFFF en cas de problme dans la transmission
*/
int Vitesse_Robot_Adverse(void)
{
	int Pos1,Pos2;												// Position lors de la premire mesure
	Pos1=I2C_Receive(ADR_I2C_TOURELLE);							// Mesure la premire position du robot
	if (Pos1==-1) return 0xFFFF;									// Traite les erreurs
	
	Delay10KTCYx(255);											// Attends avant la seconde mesure
	
	Pos2=I2C_Receive(ADR_I2C_TOURELLE);							// Ralise la seconde mesure
	if (Pos2==-1) return 0xFFFF;									// Traite les erreurs
	return ((Pos2-Pos1)%128);									// Retourne la vitesse du robot adverse
}	


// --------------------------------------------------------------------------
/* Retourne la distance avec le robot adverse
 *  entre 0 et 255 si tout s'est bien pass
 *  -1 en cas de problme dans la transmission
*/
int Distance_Robot_Adverse(void) 
{
	unsigned char Data[2];										// Donnes brutes
	int Ret,i=1;				
	do															// Valeur de retour de l'envoie sur l'I2C
		Delay100TCYx(i);
	while (I2C_Receive_x(ADR_I2C_TOURELLE,2,Data) && i<10);		// Lecture des donnes
	if (i==10)
		return -1;												// En cas d'erreur sur l'I2C, on retourne -1		
	return Data[1];		
}


// --------------------------------------------------------------------------
/* Retourne l'orientation absolue du robot adverse
 *  Valeur comprise entre -PI et +PI  
*/
float Angle_Absolu_Robot_Adverse (void)
{
	float 			Angle_Adverse;											// Angle form avec le robot adverse
	float 			Angle_Robot;											// Orientation courante du robot
		
	Angle_Adverse=Angle_Robot_Adverse();									// Mesure de l'angle du robot adverse
	Angle_Adverse=Angle_Adverse*2.0*PI/255.0;								// Conversion l'angle de l'adversaire en radians		
	
	Angle_Robot=A_Propulsion()/1000.0;										// Mesure et convertit l'angle du robot en radian
	Angle_Adverse=Angle_Robot-PI+Angle_Adverse;
	
	while (Angle_Adverse> PI)	Angle_Adverse-=2*PI;
	while (Angle_Adverse<-PI)	Angle_Adverse+=2*PI;
	
	return (Angle_Adverse);
}



// --------------------------------------------------------------------------
/* Effectue un chek-up de la tourelle
 * Vrifie qu'on ne reoit pas d'missions parasites 	(erreur 1)
 * Localise l'adversaire ... ou attends un clic			(erreur 2)
 *  ... et vrifie qu'il est du bon ct 			  	(erreur 3)
 * Pas de problme : on retourne 0 
*/
unsigned char Check_Up_Tourelle(unsigned char couleur)
{
	if (Distance_Robot_Adverse()!=0)  			return 1; 					// On voit la balise avant l'allumage
	while (!Distance_Robot_Adverse())										// On attends de voir la balise
		if (Joy_Sw())	return 2;
	//print_int(Angle_Robot_Adverse());
	if (couleur==BLEUE)															// Si on joue bleu
	{
		if (Angle_Robot_Adverse()>127 || Angle_Robot_Adverse()<95)			// L'adversaire doit tre lgrement sur la droite
			return 3;
	}
	else
	{
		if (Angle_Robot_Adverse()>159 || Angle_Robot_Adverse()<127)			// Sinon, lgrement sur la gauche
			return 3;
	}
	return 0;																// Tout s'est bien pass
	
}	

