// Fichier Strategie.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                source de la librairie             	*
 *					strategie							*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la straegie   		*
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <delays.h>
#include "mystdio.h"

#include "defense_bleue.h"
#include "strategie_bleu.h"
#include "recal.h"
#include "ejecteur.h"



int abs(int val){
	return val>0? val : -val;
}


/* test DemiTour_Conteneur
	Force_Propulsion(1400, 1900, 3142);									// Initialise la position initiale du robot
	
	DemiTour_Conteneur( 2 );
	Polaire_Propulsion( 40, -1 );
	
	while( X_Propulsion()>2000 );
	Stop_Propulsion();
	
	while(1);
*/

/** permet de faire un demi-tour (et/ou de se recaler en Y)
  * suppose qu'on est le long du conteneur et qu'on peut avancer le long de ce bord
  * Arguments :
  *		sens : 
  *			-> 1 pour 1/2 tour vers la droite (se tourner vers le ct bleu)
  *			-> -1 pour 1/2 tour vers la gauche (se tourner vers le ct rouge)
  *			-> 2 pour revenir dans le mme sens (permet de se recaler en Y)
  *			-> -2 pour changer de sens
  * Valeur renvoye :
  *		aucune
  */
void DemiTour_Conteneur(char sens){
	char sens_depart;

// XM, 02/05
	int angle, angle_old;
	int cons;
	
	if( sens==2 || sens==-2){
		// quel est le sens d'origine?
		angle = A_Propulsion();
		if( abs(angle)<1571 )
			sens_depart = -1;	// actuellement vers la gauche
		else
			sens_depart = 1;	// actuellement vers la droite
		
		// quel est le sens cible ?
		if( sens==2 )		
			sens = sens_depart;		// conserver le sens de dpart
		else 
			sens = -sens_depart;	// inverser le sens de dpart
	}

		
	Delay10KTCYx(10);
	
	// on s'oriente perpendiculairement
	Angle_Propulsion( -PI/2, 40 );
	while( Etat_Propulsion()!='a' );
	Stop_Propulsion();

	// on s'oriente perpendiculairement
	Angle_Propulsion( -PI/2, 20 );
	while( Etat_Propulsion()!='a' );
	Stop_Propulsion();
	
	// on se recale
	if (Recal_Arriere())
		Force_Propulsion( X_Propulsion(), 2060, -1571 );
	

 	// si s'orienter vers le rouge
	if( sens==CROUGE ){
	 	Direct_Propulsion( -250, 400 );
	 	while( A_Propulsion()<-450 );
		 	
	 	Stop_Propulsion();
	 	
	 	cons = 0;
	 	Direct_Propulsion(-400, cons);
	 	angle_old = 0;
	 	do{
		 	angle = A_Propulsion();
		 	if( angle_old==angle ){
			 	cons+=20;
			 	Direct_Propulsion(-400, cons);
			}
		 	angle_old = angle;
		}while( angle<0 );

	 
	 // si s'orienter vers le bleu
	}else{
		
	 	Direct_Propulsion( 400, -250 );
	 	while( A_Propulsion()>-2691 );
	 	
	 	Stop_Propulsion();
	 	
	 	cons = 0;
	 	Direct_Propulsion(cons, -400);
	 	angle_old = 0;
	 	do{
		 	angle = A_Propulsion();
		 	if( angle_old==angle ){
			 	cons+=20;
			 	Direct_Propulsion(cons, -400 );
			}
		 	angle_old = angle;
		}while( angle>-PI*1000+10 && angle<0 );	 	
	}
	
	Stop_Propulsion();
	
	if( sens==CROUGE )
		Angle_Propulsion( 0, 20 );
	else
		Angle_Propulsion( PI, 20 );
		
	while( Etat_Propulsion()!='a' );
	Stop_Propulsion();
	
}

#define VIT_POUSSE	50	// la vitesse  laquelle on pousse

/* EnleverBalles : permet d'enlever les balles adverses de sa partie de conteneur
	prrequis : on a  peu prs le distributeur dans le dos (angle robot = -pi/2)
	et qu'on en n'est pas trop loin (quelques centimtres).
	Arguments :
		couleur : la couleur DU ROBOT ADVERSE (1 pour bleu, -1 pour rouge)
	Valeur renvoye :
		0 si a s'est bien pass
		-1 si on a t bloqu et qu'on a d abandonner la manoeuvre
 */  

int EnleverBalles(char couleur ){
	int x,y;	// la position du robot
	int adv_present=0;	// indique, par une valeur de 0  3, le "degr" de prsence devant nous de l'adversaire
	int ret, temps;
	char ralenti=0;
	
	switch( couleur ){
		case ROUGE :
			couleur = CROUGE;
			break;
		case BLEUE :
			couleur = CBLEU;
			break;
		default :return -2;
	}

		
/*
 enlever les balles adverses :
	-> faire un recalage arrire
	-> faire un quart de tour en s'orientant vers le conteneur adverse
	-> si on pousse vers le rouge et que x<1500
	-> ou si on pousse vers le bleu et que x>1500
		-> avancer jusqu'au milieu ou jusqu' ce qu'on soit bloqu
	-> (ventuellement : si on pousse vers le rouge et que x>1500
						ou si on pousse vers le bleu et que x<1500
							-> reculer jusqu'au milieu ou jusqu' ce qu'on soit bloqu)
	-> dplier le bras
	-> avancer jusqu'au bout du conteneur ou jusqu' ce qu'on soit bloqu, 
		soit par l'adversaire, soit parce que a force (normalement a ne devrait pas arriver)
	-> replier le bras
*/		
		// tape 1 : on s'oriente vers le ct adverse
		DemiTour_Conteneur( couleur );	// s'orienter vers la couleur adverse
		
		//print_str("1/2 tour fait\n");
		//Joy_Click();
		
		// tape 2 : si ncessaire, on se rend au milieu pour viter de pousser nos balles
		x = X_Propulsion();
		y = Y_Propulsion();
		if( couleur==CROUGE && x<1290 ){
			GotoXY_Propulsion( 1290, y, VIT_POUSSE );
		}else if( couleur==CBLEU && x>1710 ){
			GotoXY_Propulsion( 1710, y, VIT_POUSSE );
		}
		
		while( Etat_Propulsion()!='a' ){	// on attend d'tre arriv ou bloqu
			if( Adversaire_Devant()>=2 ){
				Stop_Propulsion();			// si l'adversaire est prsent, on s'arrte
				return -1;	
			}
		}
		//Stop_Propulsion();
		
		print_str(">milieu ok\n");
		
		// on se roriente proprement avant de dployer le bras
		if( couleur==CROUGE )
			Angle_Propulsion( 0, 20 );
		else
			Angle_Propulsion( PI, 20 );
			
		ret = 0;
		temps = Sec_depuis_debut();
		while( Etat_Propulsion()!='a' ){
			if( Sec_depuis_debut()-temps>1 ){
				print_str("rate\n");
				//Joy_Click();
				ret=-2;
				break;
			}
		}
		if(ret!=-2)
		print_str(">angle ok\n");
		//Joy_Click();
		
	
		
		// tape 2bis : on se recule jusqu' tre en dea de x=1500 pour pouvoir prendre toutes les balles adverses
		if( couleur==CROUGE )
			GotoXY_Propulsion( X_Propulsion()-200, Y_Propulsion(), -VIT_POUSSE );
		else
			GotoXY_Propulsion( X_Propulsion()+200, Y_Propulsion(), -VIT_POUSSE );
					
		while( Etat_Propulsion()!='a' && Etat_Propulsion()!='g' && Adversaire_Derriere()<2);
		Stop_Propulsion();


		// tape 3 : on dploie le bras 
		if( couleur==CROUGE )
			Ejecteur_Gauche_Actif();	// on dploie le bras gauche
		else
			Ejecteur_Droit_Actif();		// on dploie le bras droit		
		Delay10KTCYx(50);				// et on attend un peu (pour que ce soit fait)

		//print_str("bras deploye\n");
		//Joy_Click();
		print_str("GotoXY:");
		
		// tape 4 : on avance jusqu'au bout du conteneur adverse
		if( couleur==CROUGE )
			GotoXY_Propulsion( 3000, y+15, VIT_POUSSE );
		else
			GotoXY_Propulsion( 0, y+15, VIT_POUSSE );
		
		while( Etat_Propulsion()!='a' ){	// on attend d'tre arriv ou bloqu
			if( Adversaire_Devant()>=2 ){
				print_str("adversaire\n");
				//Joy_Click();
				Stop_Propulsion();			// si l'adversaire est prsent, on s'arrte
				ret = -1;	
			}
			if( Etat_Propulsion()=='g' ){
				Stop_Propulsion();			// sommes-nous bloqu ?		
				print_str("bloque!\n");
				//Joy_Click();
				ret = 0;
			}
			if( !ralenti ){ 
				ralenti = 1;
				if( couleur==CROUGE && X_Propulsion()>2500 )
					GotoXY_Propulsion( 3000, y+15, 20 );
				else if( couleur==CBLEU && X_Propulsion()<500 )
					GotoXY_Propulsion( 0, y+15, 20 );

			}
		}
		
		
		if( ret>0 ){		
			print_str("ok\n");
			//Joy_Click();
			ret = 0;
		}
			
		/*
		// on met un petit angle avant de relever le bras			
		if( couleur==CBLEU )
			Angle_Propulsion(0,20);
		else
			Angle_Propulsion(PI,20);
	
		
		temps = Sec_depuis_debut();
		while( Etat_Propulsion()!='a' ){
			if( Sec_depuis_debut()-temps>1 ){
				print_str("rate\n");
				//Joy_Click();
				ret=-2;
				break;
			}
		}
		*/	

		// tape 5 : on replie le bras 
		if( couleur==CROUGE )
			Ejecteur_Gauche_Repos();	// on replie le bras gauche
		else
			Ejecteur_Droit_Repos();		// on replie le bras droit		
		Delay10KTCYx(250);				// et on attend un peu (pour que ce soit fait)
		Delay10KTCYx(250);				// et on attend un peu (pour que ce soit fait)
		
		print_str("bras replie\n");
		//Joy_Click();
		return ret;
}
	

/* PousserBalles : permet de pousser vers chez nous les balles adverses 
	prrequis : on a  peu prs le distributeur dans le dos (angle robot = -pi/2)
	et qu'on en n'est pas trop loin (quelques centimtres).
	Arguments :
		couleur : la couleur DU ROBOT ADVERSE (1 pour bleu, -1 pour rouge)
		enlever : 1 pour enlever, 0 pour pousser vers chez nous
	Valeur renvoye :
		0 si a s'est bien pass
		-1 si on a t bloqu et qu'on a d abandonner la manoeuvre
 */  

int PousserBalles( char couleur ){
	int x,y;	// la position du robot
	int adv_present=0;	// indique, par une valeur de 0  3, le "degr" de prsence devant nous de l'adversaire
	int ret, temps;
	
	switch( couleur ){
		case ROUGE :
			couleur = CROUGE;
			break;
		case BLEUE :
			couleur = CBLEU;
			break;
		default :return -2;
	}

/*
  pousser les balles adverses vers chez nous :
	on suppose qu'on a  peu prs le distributeur dans le dos (angle robot = -pi/2)
	et qu'on en n'est pas trop loin.
	-> faire un recalage arrire
	-> faire un quart de tour en s'orientant vers notre conteneur
		(utiliser la fonction DemiTour_Conteneur pour ces deux derniers points)
	-> si on pousse vers le bleu et que x est trop faible
	-> ou si on pousse vers le rouge et que x est trop grand
		-> reculer jusqu'au bout ou jusqu' ce qu'on soit bloqu
	-> dplier le bras
	-> avancer jusqu'au milieu ou jusqu' ce qu'on soit bloqu, 
		soit par l'adversaire, soit parce que a force (normalement a ne devrait pas arriver)
	-> replier le bras
*/
	DemiTour_Conteneur( -couleur );	// s'orienter vers chez nous
	
	print_str("1/2 tour fait\n");
	//Joy_Click();
	
	// tape 2 : si ncessaire, on recule pour pousser un maximum de balles adverses
	x = X_Propulsion();
	y = Y_Propulsion();
	if( couleur==CBLEU && x>1310 ){
		GotoXY_Propulsion( 880, y, -VIT_POUSSE );
	}else if( couleur==CROUGE && x<1690 ){
		GotoXY_Propulsion( 2120, y, -VIT_POUSSE );
	}
	
	while( Etat_Propulsion()!='a' ){	// on attend d'tre arriv ou bloqu
		if( Adversaire_Derriere()>=3 ){
			Stop_Propulsion();			// si l'adversaire est prsent, on s'arrte
		}
	}
	
	print_str(">milieu ok\n");
	//Joy_Click();

	// on se roriente proprement avant de dployer le bras
	if( couleur==CROUGE )
		Angle_Propulsion( PI, 30 );
	else
		Angle_Propulsion( 0, 30 );
	while( Etat_Propulsion()!='a');
	
	print_str(">angle ok\n");
	//Joy_Click();
	
	// tape 3 : on dploie le bras 
	if( couleur==CBLEU )
		Ejecteur_Gauche_Actif();	// on dploie le bras gauche
	else
		Ejecteur_Droit_Actif();		// on dploie le bras droit		
	Delay10KTCYx(50);				// et on attend un peu (pour que ce soit fait)

	print_str("bras deploye\n");
	//Joy_Click();
	
	// tape 4 : on avance jusqu'au milieu du conteneur
	if( couleur==CBLEU )
		GotoXY_Propulsion( 1360, y+5, VIT_POUSSE );
	else
		GotoXY_Propulsion( 1640, y+5, VIT_POUSSE );
	
	while( Etat_Propulsion()!='a' ){	// on attend d'tre arriv
		if( Adversaire_Devant()>=3 ){
			print_str("adversaire\n");
			//Joy_Click();
			Stop_Propulsion();			// si l'adversaire est prsent, on s'arrte
			ret = -1;	
		}
		if( Etat_Propulsion()=='g' ){
			Stop_Propulsion();			// sommes-nous bloqu ?		
			print_str("bloque!\n");
			//Joy_Click();
			ret = -1;
		}
	}
	//Joy_Click();
	
	if( ret>0 ){		
		print_str("ok\n");
		//Joy_Click();
		ret = 0;
	}
			
	print_str("au milieu\n");
	//Joy_Click();
	
	/*
	// on met un petit angle avant de relever le bras			
	if( couleur==CBLEU )
		//Angle_Propulsion(A_Propulsion()/1000.0+0.01,20);
		Angle_Propulsion(0,20);
	else
		Angle_Propulsion(PI,20);

	temps = Sec_depuis_debut();
	while( Etat_Propulsion()!='a' ){
		if( Sec_depuis_debut()-temps>1 ){
			print_str("rate\n");
			//Joy_Click();
			ret=-2;
			break;
		}
	}
	*/
	// tape 5 : on replie le bras 
	if( couleur==CBLEU )
		Ejecteur_Gauche_Repos();	// on replie le bras gauche
	else
		Ejecteur_Droit_Repos();		// on replie le bras droit		
	Delay10KTCYx(250);				// et on attend un peu (pour que ce soit fait)
	Delay10KTCYx(250);				// et on attend un peu (pour que ce soit fait)
	
	print_str("bras replie\n");
	//Joy_Click();
}









#define VIT_SUIVI				60.0
#define VIT_LENT				30.0
#define GO_CONTENEUR_BOUT			0
#define GO_CONTENEUR_INTERM			1
#define GO_CONTENEUR_MILIEU			2

//-----------------------------------------------------------------------
/* Le robot se place en dfense de notre ct en protgeant les balles
 *
 * Arguments : 
 * 	aucun
 * 
 * Paramtre retourn : 
 *	 aucun
 */
void Defense_Bleue(void)
{
	char etat;
	int	angle_adv;		// angle du robot adverse
	int adv_present = 0;
	int x,y;			// notre position en x,y
	int stopped = 0;	// indique si on est stopp (par la prsence de l'adversaire ou parce qu'on est arriv)
	char propuls;
	int xstart;
	int cons_vit;
	int xcibles[3]={600, 950, 1300};
	char deja_pousse=0;
	int ret;
		
/*  grer
	-> qu'est-ce qu'on fait si on est bloqu ???
*/
	
	
	/* l'ide est la suivante :
		-> on est plac le long de notre partie du conteneur, au bout, le nez orient vers notre conteneur
		-> Si le robot est dernire nous (entre +PI/8 et -PI/8)
			-> il veut probablement poser des balles, nous allons le taquiner un peut
			-> on s'approche soucement jusqu' lui en longeant le bord de la table pour lui laisser un espace vital rduit
			   entre nous et son conteneur
		-> Si le robot est devant nous (entre -PI-PI/8 et -PI/2)
			-> le robot est derrire ou  ct de nous, nous allons l'empcher de s'approcher du bord de la table en restant  son niveau
				-> Le but est d'tre sur le mme axe X que l'adversaire, pour cela, on vrifie l'angle de faon  avoir -PI/2 avec lui
				-> si l'adversaire est proche de -PI/2, on s'aligne doucement, sinon on se dplace vite pour atteindre cet angle
					(donc cration d'un mini asservicement)
				-> on fait attention dans ce mode de bien longer la goutire tout en restant dans une position X comprise entre 700 et 1400,
					c'est  dire rester chez nous et ne pas empiter dans la zone adverse (interdit de l'empcher de rejoindre la goutire de sa couleure
	*/
	
	
	etat = GO_CONTENEUR_MILIEU;
	stopped = 1;
	
	// sr ???
	xstart = X_Propulsion();
	
	while(1){
		
		//x = X_Propulsion();
		//angle_adv = MAngle_Robot_Adverse();		// juste pour les tests :  virer aprs
		propuls = Etat_Propulsion();
		
//		print_strint( "etat=", etat, 0 );
//		print_strint( ",stop=", stopped, 0 );
//		print_strint( ", x=", x, 1 );
		
//		Joy_Click();
//		print_strint( ", x=", x, 0 );
//		print_strint( ",a=", angle_adv, 1 );
		
		
		
				
		cons_vit = VIT_SUIVI;
		if( xcibles[etat]>xstart ){
			adv_present = Adversaire_Derriere();
			cons_vit = -cons_vit;
		}else{
			adv_present = Adversaire_Devant();
		}
			
		// si on n'est pas encore arriv l o on voulait
		if( propuls!='a' || stopped ){
			
// TODO : remplacer le 3 par un niveau de dtection (3-> contact, 2->proche, 1->on ne sait pas trop o)
//	pour viter d'crabouiller un robot qu'on ne dtecterait pas avec les tlmtres

			if( adv_present>=2 ){	// si l'adversaire est proche de nous, on s'arrte
				Stop_Propulsion();
				stopped = 1;
			}else if(stopped){				// sinon on (re)part

				print_strint( "->etat=", etat, 0 );
				print_strint( ",vit=", cons_vit, 1 );
				print_strint( "start=", xstart, 0 );
				print_strint( ",cible=", xcibles[etat], 1 );

/*	test angle absolu adverse	
while(1){
int angle_adv = (int) (Angle_Absolu_Robot_Adverse()/PI*127);
print_strint( "angle=", angle_adv, 1 );
Delay10KTCYx(50);
}
*/	
					
				GotoXY_Propulsion( xcibles[etat], 1900, cons_vit );	
				stopped = 0;	
			}
		
		// si on est dj en position, on choisit le prochain tat
		}else{
			angle_adv = Angle_Robot_Adverse();
			xstart = xcibles[etat];
			
			
			Angle_Propulsion( PI, 20 );
			switch( etat ){	
				case GO_CONTENEUR_BOUT :
					if( angle_adv>=222 || angle_adv<=16 ){	// si l'adversaire est plutt de son ct
						stopped = 1;
						etat = GO_CONTENEUR_MILIEU;
					} else if( angle_adv>=211 ){
						stopped = 1;
						etat = GO_CONTENEUR_INTERM;
					}
					break;
				
				case GO_CONTENEUR_INTERM :
					if( angle_adv>211||angle_adv<32 ){	// si l'adversaire est plutt de son ct
						stopped = 1;
						etat = GO_CONTENEUR_MILIEU;
					} else if( angle_adv>169 ){	// s'il est  peu prs  notre hauteur
						// ne pas bouger
					} else{					// s'il est de notre ct
						stopped = 1;
						etat = GO_CONTENEUR_BOUT;
					}
					break;
				
				case GO_CONTENEUR_MILIEU :
					if( (angle_adv>=222 || angle_adv<63) ){	// si l'adversaire est de son ct 
						;									// ne pas bouger
					} else if( 63<=angle_adv && angle_adv<=160 ){	// si l'adversaire est franchement de notre ct
						etat = GO_CONTENEUR_BOUT;		// on retourne au bout en catastrophe
						stopped = 1;
					} else {								// si l'adversaire est plutt  notre hauteur
						ret = -1;
						if( Nb_Depose_Adverse==0 )															// sinon (pas devant nous), on enleve
						{
							ret = EnleverBalles( ROUGE );
							print_strint("enlever=", ret, 1 );
						}
						
						Angle_Propulsion( -PI/2, 40 );
						while( Etat_Propulsion()!='a' );
												
						if( ret<0 ){
							ret= PousserBalles( ROUGE );
							print_strint("pousser=", ret, 1 );
						}
				
						etat = GO_CONTENEUR_BOUT;
						stopped = 1;
					}
					break;
					
			} // fin switch 
			
		
		} // fin si on est dj en position

	} // fin while	
	
	
} 
